import { ValueOf } from "../utils";
import { RoomPlayer, Base } from ".";
import Client from "../client";
import { languages } from "../enums";
/** Represents a room. */
export default class Room extends Base {
    /**
     * The room name.
     */
    name: string;
    /**
     * All of the {@link Player} that are in the room.
     */
    playerList: RoomPlayer[];
    /**
     * Whether or not the room is a public.
     */
    isPublic: boolean;
    /**
     * The room language.
     */
    language: ValueOf<typeof languages>;
    /**
     * Whether or not the room is a tribe house.
     */
    isTribeHouse: boolean;
    /**
     * @hidden
     */
    constructor(client: Client, isPublic: boolean, name: string, language: ValueOf<typeof languages>);
    /**
     * Get a player by the pcode or name.
     *
     * @example
     * ```js
     * const player = client.room.getPlayer('Name#0000');
     * console.log(player.look);
     * ```
     */
    getPlayer(name: string): RoomPlayer | undefined;
    getPlayer(pcode: number): RoomPlayer | undefined;
    /**
     * Removes player from playerList.
     *
     * @hidden
     */
    removePlayer(pcode: number): RoomPlayer | undefined;
    /**
     * Add player to player list
     */
    addPlayer(player: RoomPlayer): void;
    /**
     * Adds or updates the player in room playerList.
     *
     * @hidden
     */
    updatePlayer(player: RoomPlayer): RoomPlayer;
    /**
     * Sends a message to the room.
     */
    sendMessage(message: string): void;
}
