"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var iconv_lite_1 = require("iconv-lite");
/**
 * @hidden
 */
var ByteArray = /** @class */ (function () {
    /**
     * Constructor.
     *
     * @example
     * ```js
     * const packet = new ByteArray();
     * ```
     */
    function ByteArray(buff) {
        this.buffer = Buffer.isBuffer(buff)
            ? buff
            : Array.isArray(buff)
                ? Buffer.from(buff)
                : Buffer.alloc(0);
        this.writePosition = this.buffer.length;
        this.readPosition = 0;
    }
    Object.defineProperty(ByteArray.prototype, "length", {
        get: function () {
            return this.buffer.length;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ByteArray.prototype, Symbol.toStringTag, {
        get: function () {
            return "ByteArray";
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ByteArray.prototype, "bytesAvailable", {
        get: function () {
            return this.length - this.readPosition;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Expands the buffer
     */
    ByteArray.prototype.expand = function (value) {
        if (this.length - this.writePosition < value) {
            this.buffer = Buffer.concat([
                this.buffer,
                Buffer.alloc(value - (this.length - this.writePosition)),
            ]);
        }
    };
    /**
     * Adds data to the buffer.
     */
    ByteArray.prototype.write = function (data) {
        var buffer;
        if (Buffer.isBuffer(data))
            buffer = data;
        else if (Array.isArray(data) || typeof data === "string" || data instanceof String)
            buffer = Buffer.from(data);
        else if (data instanceof ByteArray)
            buffer = Buffer.from(data.buffer);
        else
            throw new Error("The value type must be buffer, bytearray, string or array");
        this.buffer = Buffer.concat([this.buffer, buffer]);
        this.writePosition += buffer.length;
        return this;
    };
    /**
     * Clears the contents of the bytearray and resets the length and positions properties to 0.
     */
    ByteArray.prototype.clear = function () {
        this.buffer = Buffer.alloc(0);
        this.writePosition = 0;
        this.readPosition = 0;
    };
    /**
     * Reads a Boolean value from the byte stream.
     */
    ByteArray.prototype.readBoolean = function () {
        return this.readByte() !== 0;
    };
    /**
     * Reads a signed byte from the byte stream.
     */
    ByteArray.prototype.readByte = function () {
        return this.buffer.readInt8(this.readPosition++);
    };
    /**
     * Reads a signed 32-bit integer from the byte stream.
     */
    ByteArray.prototype.readInt = function () {
        var value = this.buffer.readInt32BE(this.readPosition);
        this.readPosition += 4;
        return value;
    };
    /**
     * Reads a signed 16-bit integer from the byte stream.
     */
    ByteArray.prototype.readShort = function () {
        var value = this.buffer.readInt16BE(this.readPosition);
        this.readPosition += 2;
        return value;
    };
    /**
     * Reads an unsigned byte from the byte stream.
     */
    ByteArray.prototype.readUnsignedByte = function () {
        return this.buffer.readUInt8(this.readPosition++);
    };
    /**
     * Reads an unsigned 32-bit integer from the byte stream.
     */
    ByteArray.prototype.readUnsignedInt = function () {
        var value = this.buffer.readUInt32BE(this.readPosition);
        this.readPosition += 4;
        return value;
    };
    /**
     * Reads an unsigned 16-bit integer from the byte stream.
     */
    ByteArray.prototype.readUnsignedShort = function () {
        var value = this.buffer.readUInt16BE(this.readPosition);
        this.readPosition += 2;
        return value;
    };
    /**
     * Reads a UTF-8 string from the byte stream.
     */
    ByteArray.prototype.readUTF = function () {
        var size = this.readUnsignedShort();
        var value = this.buffer.subarray(this.readPosition, this.readPosition + size).toString();
        this.readPosition += size;
        return value;
    };
    ByteArray.prototype.toJSON = function () {
        return Object.assign({}, this.buffer.toJSON().data);
    };
    /**
     * Converts the byte array to a string.
     */
    ByteArray.prototype.toString = function () {
        return iconv_lite_1.decode(this.buffer, "utf-8");
    };
    /**
     * Writes a Boolean value.
     */
    ByteArray.prototype.writeBoolean = function (value) {
        return this.writeByte(value ? 1 : 0);
    };
    /**
     * Writes a byte to the byte stream.
     */
    ByteArray.prototype.writeByte = function (value) {
        this.expand(1);
        this.buffer.writeInt8(value, this.writePosition++);
        return this;
    };
    /**
     * Writes a sequence of length bytes from the specified byte array, bytes, starting offset bytes into the byte stream.
     */
    ByteArray.prototype.writeBytes = function (bytes, offset, length) {
        if (offset === void 0) { offset = 0; }
        if (length === void 0) { length = 0; }
        if (length === 0)
            length = bytes.length - offset;
        this.expand(length);
        for (var i = 0; i < length; i++)
            this.buffer[i + this.writePosition] = bytes.buffer[i + offset];
        this.writePosition += length;
        return this;
    };
    /**
     * Writes a 32-bit signed integer to the byte stream.
     */
    ByteArray.prototype.writeInt = function (value) {
        this.expand(4);
        this.buffer.writeInt32BE(value, this.writePosition);
        this.writePosition += 4;
        return this;
    };
    /**
     * Writes a 16-bit integer to the byte stream.
     */
    ByteArray.prototype.writeShort = function (value) {
        this.expand(2);
        this.buffer.writeInt16BE(value, this.writePosition);
        this.writePosition += 2;
        return this;
    };
    /**
     * Writes a unsigned byte to the byte stream.
     */
    ByteArray.prototype.writeUnsignedByte = function (value) {
        this.expand(1);
        this.buffer.writeUInt8(value, this.writePosition++);
        return this;
    };
    /**
     * Writes a 32-bit unsigned integer to the byte stream.
     */
    ByteArray.prototype.writeUnsignedInt = function (value) {
        this.expand(4);
        this.buffer.writeUInt32BE(value, this.writePosition);
        this.writePosition += 4;
        return this;
    };
    /**
     * Writes a 16-bit unsigned integer to the byte stream.
     */
    ByteArray.prototype.writeUnsignedShort = function (value) {
        this.expand(2);
        this.buffer.writeUInt16BE(value, this.writePosition);
        this.writePosition += 2;
        return this;
    };
    /**
     * Writes a UTF-8 string to the byte stream.
     */
    ByteArray.prototype.writeUTF = function (value) {
        var size = Buffer.byteLength(value);
        this.writeUnsignedShort(size);
        this.write(Buffer.from(value, "utf8"));
        return this;
    };
    ByteArray.prototype.XXTEA = function (v, n, k) {
        var cycles = parseInt((6 + 52 / n).toString());
        var sum = BigInt(0);
        var z = v[n - 1];
        for (var i = 0; i < cycles; i++) {
            sum = (sum + BigInt(0x9e3779b9)) & BigInt(0xffffffff);
            var e = (sum >> BigInt(2)) & BigInt(3);
            for (var p = 0; p < n; p++) {
                var y = v[parseInt(((BigInt(p) + BigInt(1)) % BigInt(n)).toString())];
                z = v[p] = parseInt(((BigInt(v[p]) +
                    ((((BigInt(z) >> BigInt(5)) ^ (BigInt(y) << BigInt(2))) +
                        ((BigInt(y) >> BigInt(3)) ^ (BigInt(z) << BigInt(4)))) ^
                        ((BigInt(sum) ^ BigInt(y)) +
                            (BigInt(k[parseInt(((BigInt(p) & BigInt(3)) ^ BigInt(e)).toString())]) ^
                                BigInt(z))))) &
                    BigInt(0xffffffff)).toString());
            }
        }
        for (var p = 0; p < n; p++)
            v[p] = +v[p];
        return v;
    };
    /**
     * Cipher the packet with the XXTEA algorithm.
     */
    ByteArray.prototype.blockCipher = function (keys) {
        var st = __spread(this.buffer);
        while (this.length < 8) {
            st.push(0);
        }
        var packet = new ByteArray(st);
        var chunks = [];
        var length = packet.length;
        for (var i = 0; i < 4 - (length % 4); i++) {
            packet.writeByte(0);
        }
        while (length > 0) {
            chunks.push(packet.readUnsignedInt());
            length -= 4;
        }
        packet = new ByteArray();
        chunks = this.XXTEA(chunks, chunks.length, keys);
        packet.writeShort(chunks.length);
        for (var i = 0; i < chunks.length; i++) {
            var chunk = chunks[i];
            packet.writeUnsignedInt(+chunk);
        }
        return packet;
    };
    /**
     * Cipher the packet with the XOR algorithm.
     */
    ByteArray.prototype.xorCipher = function (keys, fingerprint) {
        var p = [];
        for (var i = 0; i < this.buffer.length; i++) {
            var byte = this.buffer[i];
            fingerprint++;
            p[i] = (byte ^ keys[fingerprint % 20]) & 255;
        }
        return new ByteArray(p);
    };
    return ByteArray;
}());
exports.default = ByteArray;
